#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-29
#
# 2016-07-19: monitor Check Point FW Log server status
# 2016-07-24: changed for check state
# 2018-01-05: fix inventory function
# 2018-03-08: changed snmp scan function and inventory function
# 2018-03-15: code cleanup
# 2020-06-08: changed snmp-scan function
# 2021-06-14: rewrite for cmk 2.0, added wato
# 2021-08-09: fixed empty string_table, fixed log server default state
# 2021-08-16: code cleanup
# 2023-04-23: refactoring

#
# sample snmpwalk
#
# CHECKPOINT-MIB::fwLSConnOverall.0 = INTEGER: 0
# CHECKPOINT-MIB::fwLSConnOverallDesc.0 = STRING: Security Gateway is reporting logs as defined
# CHECKPOINT-MIB::index.1.0 = Gauge32: 1
# CHECKPOINT-MIB::index.2.0 = Gauge32: 2
# CHECKPOINT-MIB::name.1.0 = STRING: 10.140.2.203
# CHECKPOINT-MIB::name.2.0 = STRING: 10.140.2.103
# CHECKPOINT-MIB::state.1.0 = Gauge32: 0
# CHECKPOINT-MIB::state.2.0 = Gauge32: 2
# CHECKPOINT-MIB::state_desc.1.0 = STRING: Log-Server Connected
# CHECKPOINT-MIB::state_desc.2.0 = STRING: Log-Server Disconnected
# CHECKPOINT-MIB::send_rate.1.0 = Gauge32: 0
# CHECKPOINT-MIB::send_rate.2.0 = Gauge32: 0
# CHECKPOINT-MIB::fwLocalLoggingDesc.0 = STRING: Logs are written to log server
# CHECKPOINT-MIB::fwLocalLoggingStat.0 = INTEGER: 0
# CHECKPOINT-MIB::local_logging_write_rate.0 = Gauge32: 0
# CHECKPOINT-MIB::logging_handling_rate.0 = Gauge32: 0
#
# .1.3.6.1.4.1.2620.1.1.30.1.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.1.30.2.0 = STRING: "Security Gateway is reporting logs as defined"
# .1.3.6.1.4.1.2620.1.1.30.3.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.1.30.3.1.1.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.1.30.3.1.2.1.0 = STRING: "10.140.2.203"
# .1.3.6.1.4.1.2620.1.1.30.3.1.2.2.0 = STRING: "10.140.2.103"
# .1.3.6.1.4.1.2620.1.1.30.3.1.3.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.3.1.3.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.1.30.3.1.4.1.0 = STRING: "Log-Server Connected"
# .1.3.6.1.4.1.2620.1.1.30.3.1.4.2.0 = STRING: "Log-Server Disconnected"
# .1.3.6.1.4.1.2620.1.1.30.3.1.5.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.3.1.5.2.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.4.0 = STRING: "Logs are written to log server"
# .1.3.6.1.4.1.2620.1.1.30.5.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.1.30.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.7.0 = Gauge32: 0
#
#
# sample info
# log server running
# [[['0', 'Security Gateway is reporting logs as defined', 'Logs are written to log server', '0']],
#  [['1', '192.168.10.10', '0', 'Log-Server Connected'],
#   ['2', '192.168.10.11', '2', 'Backup Log-Server Not Active']]]
#
# no log server
# [[], []]
#

from typing import NamedTuple, List, Dict, Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    check_levels,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


class CheckPointFwLsOverAll(NamedTuple):
    conn_over_all: int
    conn_over_all_desc: str
    local_logging_desc: str
    local_logging_stat: int
    local_logging_write_rate: Optional[int]
    logging_handling_rate: Optional[int]


class CheckPointFwLs(NamedTuple):
    index: int
    name: str
    state: int
    state_desc: str
    send_rate: Optional[int]


_state_desc = {
    0: 'Ok',
    1: 'Error',
    2: 'Not Active',
}

_state = {
    0: 'Logging to log servers',
    1: 'Logging local configured',
    2: 'Logging local due to connectivity',
    3: 'Logging local due to high rate',
}


def parse_checkpoint_fw_ls(string_table: List[StringTable]) -> Optional[Dict[str, any]]:
    over_all, log_servers = string_table
    try:
        conn_over_all, conn_over_all_desc, local_logging_desc, local_logging_stat, local_logging_write_rate, \
        logging_handling_rate = over_all[0]
    except (IndexError, ValueError):
        return

    parsed = {}
    parsed.update({'over all': CheckPointFwLsOverAll(
        conn_over_all=int(conn_over_all),
        conn_over_all_desc=conn_over_all_desc,
        local_logging_desc=local_logging_desc,
        local_logging_stat=int(local_logging_stat),
        local_logging_write_rate=int(local_logging_write_rate) if local_logging_write_rate.isdigit() else None,
        logging_handling_rate=int(logging_handling_rate) if logging_handling_rate.isdigit() else None,
    )})

    for index, name, state, state_desc, send_rate in log_servers:
        parsed.update({name: CheckPointFwLs(
            index=int(index),
            name=name,
            state=int(state),
            state_desc=state_desc,
            send_rate=int(send_rate) if send_rate.isdigit() else None,
        )})

        return parsed


def discovery_checkpoint_fw_ls(section: Dict[str, any]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_fw_ls(item, params, section: Dict[str, any]) -> CheckResult:
    if item == 'over all':
        over_all = section['over all']

        yield Result(state=State(over_all.conn_over_all), summary=over_all.conn_over_all_desc)
        if over_all.local_logging_desc != '':
            if over_all.local_logging_stat in [1, 3]:
                yield Result(state=State.WARN, summary=over_all.local_logging_desc)
            elif over_all.local_logging_stat == 2:
                yield Result(state=State.CRIT, summary=over_all.local_logging_desc)
            else:
                yield Result(state=State.OK, notice=over_all.local_logging_desc)

        for key, value, label in [
            ('checkpoint_fw_ls_logginghandlingrate', over_all.logging_handling_rate, 'Logging handling rate'),
            ('checkpoint_fw_ls_localloggingwriterate', over_all.local_logging_write_rate, 'Local logging write rate'),
        ]:
            if value is not None:
                yield from check_levels(
                    value=value,
                    label=label,
                    metric_name=key,
                    render_func=lambda v: f'{v}/s',
                    boundaries=(0, None),
                )

    else:
        try:
            log_server = section[item]
        except KeyError:
            return

        if log_server.state == 1:
            yield Result(state=State.CRIT, summary='State: Connection error')
        else:
            yield Result(state=State.OK, summary=f'State: {log_server.state_desc}')

        if log_server.state_desc.lower() != params["exp_connection_status"].lower():
            yield Result(state=State(params["mon_connection_state"]),
                         summary=f'Expected connection state: {params["exp_connection_status"]}')

        if log_server.send_rate is not None:  # R80.10 and up
            yield from check_levels(
                value=log_server.send_rate,
                label='Sending rate',
                metric_name='checkpoint_fw_ls_lsconnsendrate',
                render_func=lambda v: f'{v}/s',
                boundaries=(0, None),
            )


register.snmp_section(
    name='checkpoint_fw_ls',
    parse_function=parse_checkpoint_fw_ls,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.1.30',  # CHECKPOINT-MIB:fwLSConn
            oids=[
                '1',  # fwLSConnOverall
                '2',  # fwLSConnOverallDesc
                '4',  # fwLocalLoggingDesc
                '5',  # fwLocalLoggingStat
                '6',  # local_logging_write_rate
                '7',  # logging_handling_rate
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.1.30.3.1',  # CHECKPOINT-MIB::fwLSConnEntry
            oids=[
                '1',  # index
                '2',  # name
                '3',  # state
                '4',  # state_desc
                '5',  # send_rate
            ]
        )

    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_fw_ls',
    service_name='FW Log server %s',
    discovery_function=discovery_checkpoint_fw_ls,
    check_function=check_checkpoint_fw_ls,
    check_default_parameters={
        'exp_connection_status': 'Log-Server Connected',
        'mon_connection_state': 1,
    },
    check_ruleset_name='checkpoint_fw_ls',
)
